@@ -16,6 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with hgwebplus.  If not, see <https://www.gnu.org/licenses/>.
 
+import os.path
 import re
 
 from hashlib import md5 as hashlib_md5
@@ -28,9 +29,11 @@ from mercurial import (
     registrar,
     templateutil
 )
-from mercurial.hgweb import webutil, hgweb_mod
+from mercurial.hgweb import webutil, hgweb_mod, hgwebdir_mod
 from mercurial.hgweb.common import paritygen
 
+from urllib.parse import urlparse
+
 import mistune
 
 from mistune.directives import DirectiveToc
@@ -251,8 +254,19 @@ def http_clone_url(context, mapping):
     """
 
     ui = context.resource(mapping, b'ui')
+    repo = context.resource(mapping, b'repo')
 
-    return ui.config(b'web', b'http_clone_url', b'')
+    uri = ui.config(b'web', b'http_clone_url', b'')
+    if uri == b'':
+        base = ui.config(b'web', b'http_base_url', b'')
+
+        if base != '':
+            if base[-1] != b'/':
+                base += b'/'
+
+            return base + os.path.relpath(repo.root.decode('UTF-8')).encode()
+
+    return b''
 
 
 def readme(context, mapping):
@@ -265,7 +279,8 @@ def readme(context, mapping):
 
     class Renderer(mistune.HTMLRenderer):
         def image(self, src, alt, title):
-            src = 'rawfile/%s/%s' % (ctx.rev(), src)
+            if urlparse(src).netloc == '':
+                src = 'rawfile/%s/%s' % (ctx.rev(), src)
 
             return super().image(src, alt, title)
 
@@ -308,25 +323,18 @@ def ssh_clone_url(context, mapping):
     return ui.config(b'web', b'ssh_clone_url', b'')
 
 
-def sub_title(context, mapping):
-    """ sub_title is a template keyword that outputs the configuration value of
-        `web.sub_title`.  Themes can use this as a secondary text to the title.
-    """
+def dir_custom_templater(orig, self, req, nonce):
+    templater = orig(self, req, nonce)
 
-    ui = context.resource(mapping, b'ui')
+    templatekeyword = registrar.templatekeyword(templater._proc._defaults)
 
-    return ui.config(b'web', b'sub_title', b'')
+    sub_title = lambda c, m: self.ui.config(b'web', b'sub_title', b'')
+    templatekeyword(b'sub_title', requires=())(sub_title)
 
+    title = lambda c, m: self.ui.config(b'web', b'title', b'')
+    templatekeyword(b'title', requires=())(title)
 
-def title(context, mapping):
-    """ title is a template keyword that outputs the configuration value of
-        `web.title`.  Themes can use this as the HTML title and else where on
-        the pages.
-    """
-
-    ui = context.resource(mapping, b'ui')
-
-    return ui.config(b'web', b'title', b'')
+    return templater
 
 
 def repo_custom_templater(orig, self, req):
@@ -339,11 +347,15 @@ def repo_custom_templater(orig, self, req):
 
     templatekeyword = registrar.templatekeyword(templater._proc._defaults)
 
-    templatekeyword(b'http_clone_url', requires={b'ui'})(http_clone_url)
+    templatekeyword(b'http_clone_url', requires={b'ui', b'repo'})(http_clone_url)
     templatekeyword(b'readme', requires={b'ctx', b'repo'})(readme)
-    templatekeyword(b'ssh_clone_url', requires={b'ui'})(ssh_clone_url)
-    templatekeyword(b'sub_title', requires={b'ui'})(sub_title)
-    templatekeyword(b'title', requires={b'ui'})(title)
+    templatekeyword(b'ssh_clone_url', requires={b'ui', b'repo'})(ssh_clone_url)
+
+    sub_title = lambda c, m: self.config(b'web', b'sub_title', b'')
+    templatekeyword(b'sub_title', requires=())(sub_title)
+
+    title = lambda c, m: self.config(b'web', b'title', b'')
+    templatekeyword(b'title', requires=())(title)
 
     return templater
 
@@ -352,7 +364,12 @@ def extsetup(ui):
     extensions.wrapfunction(webutil, '_diffstattmplgen', _diffstattmplgen)
     extensions.wrapfunction(webutil, '_diffsgen', _diffsgen)
 
-    # this nasty but amazingly working hack is from av6
+    # these nasty but amazingly working hacks are from av6
+    extensions.wrapfunction(
+        hgwebdir_mod.hgwebdir,
+        'templater',
+        dir_custom_templater,
+    )
     extensions.wrapfunction(
         hgweb_mod.requestcontext,
         'templater',
